from django.db import models
from django.contrib.auth.models import User
from django.conf import settings
from datetime import datetime
from comments.models import Comment
from django.db.models.signals import post_save, pre_save
from django.db.models import F, Count
from ratings.models import *
from duckmarks.models import *
from PIL import Image
from images import *
from StringIO import StringIO
from django.core.files.uploadedfile import InMemoryUploadedFile
import settings

"""
#########################################################################
TUPLES
#########################################################################
"""

COMIC_RATINGS = (
    ('E', 'Everyone'),
    ('T', 'Teen'),
    ('M', 'Mature'),
    ('A', 'Adult'),
)

COMIC_TYPES = (
    (0, 'Comic Strip'),
    (1, 'Comic Book/Story'),
)

COMIC_GENRES = (
    (0, 'Fantasy'),
    (1, 'Parody'),
    (2, 'Real Life'),
    (4, 'Sci-Fi'),
    (5, 'Horror'),
    (6, 'Abstract'),
    (8, 'Adventure'),
    (9, 'Noir'),
    (12, 'Political'),
    (13, 'Spiritual'),
    (14, 'Romance'),
    (15, 'Superhero'),
    (16, 'Western'),
    (17, 'Mystery'),
    (18, 'War'),
    (19, 'Tribute'),
)

COMIC_STYLES = (
    (0, 'Cartoon'),
    (1, 'American'),
    (2, 'Manga'),
    (3, 'Realism'),
    (4, 'Sprite'),
    (5, 'Sketch'),
    (6, 'Experimental'),
    (7, 'Photographic'),
    (8, 'Stick Figure'),
)

COMIC_TONES = (
    (0, 'Comedy'),
    (1, 'Drama'),
    (3, 'Other'),
)

FILE_TYPES = (
    ('jpg', 'jpg'),
    ('png', 'png'),
    ('gif', 'gif'),
    ('tif', 'tif'),
)

COMIC_LINK_TYPES = (
    (0, 'header links'),
    (1, 'other comics'),
    (2, 'misc links'),
)


"""
#########################################################################
MANAGERS
#########################################################################
"""

class LiveComicManager(models.Manager):
    
    def get_query_set(self):
    
        return super(LiveComicManager, self).get_query_set().filter(is_deleted=False, frozen=False, date_modified__isnull=False)

"""
#########################################################################
POST IMAGE HANDLERS
#########################################################################
"""

def comic_thumbnail_upload(instance, filename):
    
    comic_name = instance.title.replace(' ', '_')
    thumbnail_path = settings.AWS_ROOT + instance.user.username + '/comics/' + comic_name + '/thumb/' + filename
    return thumbnail_path

def comic_title_upload(instance, filename):
    
    comic_name = instance.title.replace(' ', '_')
    title_path = settings.AWS_ROOT + instance.user.username + '/comics/' + comic_name + '/layout/' + filename
    return title_path

def comic_featured_upload(instance, filename):

    comic_name = instance.comic.title.replace(' ', '_')
    featured_path = 'features/' + comic_name + '/' + filename
    return featured_path

def comic_page_upload(instance, filename):
    
    comic_name = instance.comic.title.replace(' ', '_')
    save_directory = settings.AWS_ROOT + instance.comic.user.username  + '/comics/' + comic_name + '/web/' + filename
    return save_directory

def link_image_upload(instance, filename):
    
    comic_name = instance.comic.title.replace(' ', '_')
    save_directory = settings.AWS_ROOT + instance.comic.user.username  + '/comics/' + comic_name + '/layout/' + filename
    return save_directory

"""
#########################################################################
PRE IMAGE HANDLERS
#########################################################################
"""

def page_cropper(sender, **kwargs):

    try:
        im = Image.open(kwargs['instance'].content)
    except:
        return    
    im, resized = rescale(im, 960)
    io = StringIO()
    filename = kwargs['instance'].content.name.split('/')[-1]
    ext = filename.split('.')[-1].lower()
    if ext == 'jpg':
        ext = 'jpeg'
    im.save(io, format=ext.upper(), quality=100)
    file = InMemoryUploadedFile(io, None, filename, 'image/' + ext, io.len, None)
    kwargs['instance'].content = file
    
def header_cropper(sender, **kwargs):

    try:
        im = Image.open(kwargs['instance'].header)
    except:
        return    
    im, resized = rescale(im, 960)
    io = StringIO()
    filename = kwargs['instance'].header.name.split('/')[-1]
    ext = filename.split('.')[-1].lower()
    if ext == 'jpg':
        ext = 'jpeg'
    im.save(io, format=ext.upper(), quality=100)
    file = InMemoryUploadedFile(io, None, 'header.' + ext, 'image/' + ext, io.len, None)
    kwargs['instance'].header = file

def thumbnail_cropper(sender, **kwargs):

    try:
        im = Image.open(kwargs['instance'].thumbnail)
    except:
        return    
    im, resized = rescale(im, 80, 100, True)
    io = StringIO()
    filename = kwargs['instance'].thumbnail.name.split('/')[-1]
    ext = filename.split('.')[-1].lower()
    if ext == 'jpg':
        ext = 'jpeg'
    im.save(io, format=ext.upper(), quality=100)
    file = InMemoryUploadedFile(io, None, 'thumb.' + ext, 'image/' + ext, io.len, None)
    kwargs['instance'].thumbnail = file
    
def feature_cropper(sender, **kwargs):

    try:
        im = Image.open(kwargs['instance'].thumbnail)
    except:
        return
    im, resized = rescale(im, 105, 160, True)
    im = im.convert('RGB')
    io = StringIO()
    filename = kwargs['instance'].thumbnail.name.split('/')[-1]
    ext = filename.split('.')[-1].lower()
    if ext != 'jpg' and ext != 'jpeg':
        im.save(io, format='JPEG', quality=95)
    else:
        im.save(io, format='JPEG')
    file = InMemoryUploadedFile(io, None, 'feature.jpg', 'image/jpeg', io.len, None)
    kwargs['instance'].thumbnail = file

def link_image_cropper(sender, **kwargs):

    try:
        im = Image.open(kwargs['instance'].image)
    except:
        return
    im, resized = rescale(im, 150)
    io = StringIO()
    filename = kwargs['instance'].image.name.split('/')[-1]
    ext = filename.split('.')[-1]
    if ext == 'jpg':
        ext = 'jpeg'
    im.save(io, format=ext.upper())
    file = InMemoryUploadedFile(io, None, filename, 'image/' + ext, io.len, None)
    kwargs['instance'].image = file

"""
#########################################################################
MODELS
#########################################################################
"""

from django.core.mail import send_mail

class Comic(models.Model):
   
    user = models.ForeignKey(User, related_name='comics', null=True, blank=True)
    is_deleted = models.BooleanField(default=False)
    title = models.CharField(max_length=255, unique=True)
    rating = models.CharField(choices=COMIC_RATINGS, max_length=1, default='E')
    type = models.IntegerField(choices=COMIC_TYPES, default=1)
    genre = models.IntegerField(choices=COMIC_GENRES, default=1)
    tone = models.IntegerField(choices=COMIC_TONES, default=1)
    style = models.IntegerField(choices=COMIC_STYLES, default=1)
    description = models.TextField()
    thumbnail = models.ImageField(upload_to=comic_thumbnail_upload, null=True, blank=True, max_length=750)
    header = models.ImageField(blank=True, null=True, upload_to=comic_title_upload, max_length=750)
    frozen = models.BooleanField(default=False, blank=True)
    enable_forums = models.BooleanField(default=True)
    enable_homepage = models.BooleanField(default=False)
    date_created = models.DateTimeField(auto_now_add=True)
    date_modified = models.DateTimeField(null=True)
    
    # i want to get rid of this eventually
    seven_day_visits = models.IntegerField(default=0)

    assistants = models.ManyToManyField(User, related_name="assisted_comics", null=True, blank=True)
    slug = models.SlugField(max_length=255, blank=True,null=True)
    favorites = models.ManyToManyField(User, related_name="favorite_comics", null=True, blank=True)
    recommends = models.ManyToManyField(User, related_name="recommended_comics", null=True, blank=True)
    likes = models.ManyToManyField(Like, null=True, blank=True)
    email_alerts = models.ManyToManyField(User, related_name="email_alerts", null=True, blank=True)
    duckmarks = models.ManyToManyField(Duckmark, null=True, blank=True)
    
    objects = models.Manager()  # Declare this manager first to use default manager for admin
    live = LiveComicManager()

    class Meta:
        ordering = ['title']
        verbose_name_plural = "Comics"
        verbose_name = "Comic"

    def __unicode__(self):
        return self.title

class ComicCredit(models.Model):
    
    comic = models.ForeignKey(Comic, related_name='credits')
    credit = models.CharField(max_length=255)
    name = models.CharField(max_length=255)

    class Meta:
    
        verbose_name = "Comic Credit"
        verbose_name_plural = "Comic Credits"

    def __unicode__(self):
        
        return self.credit, self.name

class FeaturedComicManager(models.Manager):
    def get_by_natural_key(self, title):
        return self.get(title=self.comic.title)

class FeaturedComicForFeed(models.Model):
    objects = FeaturedComicManager()
    
    comic = models.OneToOneField(Comic, related_name='feed_feature')
    thumbnail = models.ImageField(upload_to=comic_featured_upload, blank=True, null=True, max_length=750)
    description = models.TextField()
    date_featured = models.DateTimeField(default=datetime.now())

    def natural_key(self):
        return self.comic.title

    class Meta:
        ordering = ['comic']
        verbose_name = 'Feed Featured Comic'
        verbose_name_plural = 'Feed Featured Comics'

    def __unicode__(self):
        return self.comic.title

class FeaturedComic(models.Model):

    comic = models.OneToOneField(Comic, related_name='featured')
    thumbnail = models.ImageField(upload_to=comic_featured_upload, blank=True, null=True, max_length=750)
    description = models.TextField()
    date_featured = models.DateTimeField(default=datetime.now())

    class Meta:
        ordering = ['comic']
        verbose_name = 'Featured Comic'
        verbose_name_plural = 'Featured Comics'

    def __unicode__(self):
        return self.comic.title


from django.core.mail import EmailMultiAlternatives
from django.utils.html import strip_tags
from django.template.loader import render_to_string

class ComicPage(models.Model):
    
    comic = models.ForeignKey(Comic, related_name='pages', null=True)
    is_deleted = models.BooleanField(default=False)
    post_date = models.DateTimeField(default=datetime.now())
    title = models.CharField("page title", max_length=255)
    is_chapter = models.BooleanField("is Chapter?",default=False, blank=True)
    comment = models.TextField(null=True, blank=True)
    content = models.ImageField(upload_to=comic_page_upload, null=True, blank=True, max_length=750)
    date_added = models.DateTimeField(auto_now_add=True)
    date_modified = models.DateTimeField(auto_now=True)
    order_id = models.IntegerField("sort order",max_length=3, null=True, blank=True, default=0)
    comments = models.ManyToManyField(Comment, null=True, blank=True)
    old_page_id = models.IntegerField(null=True, blank=True)
    comic_name = models.CharField(max_length=255, null=True)
    file_ext = models.CharField(max_length=5, null=True)
    
    def save(self, *args, **kwargs):
        
        id = self.id
        super(ComicPage, self).save(*args, **kwargs)
        if not id and self.post_date <= datetime.now():
            self.comic.date_modified = datetime.now()
            self.comic.save()
            self.alert_users()

    class Meta:
        
        verbose_name_plural = "Comic Pages"
        verbose_name = "Comic Page"

    def __unicode__(self):
        
        return self.title
        
    def get_absolute_url(self):
        
        return settings.COMIC_URL + self.comic + "/web/" + self.filename
    
    def alert_users(self):
        
        from_email = 'updates@drunkduck.com'
        for user in self.comic.email_alerts.all():
            try:
                subject = "%s has updated on The Duck Webcomics!" % self.comic.title
                to = user.email
                html_content = render_to_string('emails/comic_update.html', {'user':user, 'page': self})
                text_content = strip_tags(html_content) # this strips the html, so people will have the text as well.
                # create the email, and attach the HTML version as well.
                msg = EmailMultiAlternatives(subject, text_content, from_email, [to])
                msg.attach_alternative(html_content, "text/html")
                msg.send()
            except:
                continue

class ComicLink(models.Model):
    
    comic = models.ForeignKey(Comic, related_name='links')
    name = models.CharField(max_length=255)
    url = models.URLField(max_length=750)
    type = models.IntegerField(choices=COMIC_LINK_TYPES, default=0)
    image = models.ImageField(upload_to=link_image_upload, null=True, blank=True, max_length=750)
    
    class Meta:
        verbose_name_plural = "Comic Links"
        verbose_name = "Comic Link"
        
    def __unicode__(self):
        
        return self.comic.title + ' ' + self.name
    
"""
#########################################################################
CONNECTORS
#########################################################################
"""

pre_save.connect(link_image_cropper, sender=ComicLink, dispatch_uid='update_comic_link')    
pre_save.connect(thumbnail_cropper, sender=Comic, dispatch_uid='update_comic_thumbnail')
pre_save.connect(feature_cropper, sender=FeaturedComic, dispatch_uid='update_feature')
pre_save.connect(header_cropper, sender=Comic, dispatch_uid='update_comic_header')
pre_save.connect(page_cropper, sender=ComicPage, dispatch_uid='update_page')
