from datetime import datetime
import mimetypes
import os
from boto.s3.connection import S3Connection
from boto.s3.key import Key
from django.contrib.auth.decorators import login_required
from django.contrib.contenttypes.models import ContentType
from django.conf import settings
from django.forms.formsets import formset_factory
from django.http import HttpResponseRedirect, HttpResponse, HttpResponseNotFound
from django.shortcuts import render_to_response, get_object_or_404, render
from django.template import RequestContext
from django import forms as forms
from comics.models import *
from comics.forms import *
from django.template.defaultfilters import slugify
from users.templatetags.user_tags import is_mature, can_see_comic
from requests.models import *
from layouts.models import *
from layouts.forms import *
from django.db.models import Max

@login_required
def drop_assistant(request, id):
	
	comic = get_object_or_404(Comic, pk=id)
	comic.assistants.remove(request.user)
	return HttpResponseRedirect('/user/#assists')

@login_required
def manage_pages(request, id):
	
	comic = get_object_or_404(Comic, pk=id)
	if not comic.user == request.user and not request.user in comic.assistants.all() and not request.user.is_staff:
		return HttpResponseNotFound()
	pages = comic.pages.filter(is_deleted=False).order_by('-order_id')
	pages_list = list(pages)
	if request.method == 'POST':
		if request.POST.get('delete', False):
			page = ComicPage.objects.get(pk=request.POST['page_id'])
			page.is_deleted = True
			page.save()
		elif request.POST.get('move_up', False):
			page = ComicPage.objects.get(pk=request.POST['page_id'])
			page_index = pages_list.index(page)
			new_page = pages[page_index-1]
			temp = new_page.order_id
			new_page.order_id = page.order_id
			page.order_id = temp
			page.save()
			new_page.save()
                elif request.POST.get('move_down', False):
			page = ComicPage.objects.get(pk=request.POST['page_id'])
			page_index = pages_list.index(page)
			new_page = pages[page_index+1]
			temp = new_page.order_id
			new_page.order_id = page.order_id
			page.order_id = temp
			page.save()
			new_page.save()
	pages = comic.pages.filter(is_deleted=False).order_by('-order_id')
	variables = {
		'comic': comic,
		'pages': pages
	}
	return render_to_response('comics/manage_pages.html', variables, context_instance=RequestContext(request))

@login_required
def manage_credits(request, id):
	
	comic = get_object_or_404(Comic, pk=id)
	if comic.user != request.user and request.user not in comic.assistants.all():
		return HttpResponseNotFound()
	if request.method == 'POST':
		if request.POST.get('add', False):
			credit = ComicCredit.objects.create(comic=comic, credit=request.POST['type'], name=request.POST['name'])
		elif request.POST.get('delete', False):
			credit = ComicCredit.objects.get(pk=request.POST['credit_id'])
			credit.delete()
	variables = {
		'comic': comic,
		'credits': comic.credits.all(),
	}
	return render_to_response('comics/manage_credits.html', variables, context_instance=RequestContext(request))

@login_required
def manage_assistants(request, id):
	
	error = False
	comic = get_object_or_404(Comic, pk=id)
	comic_type = ContentType.objects.get_for_model(comic)
	if comic.user != request.user:
		return HttpResponseNotFound()
	if request.method == 'POST':
		user = User.objects.filter(username__iexact=request.POST['username'])
		if user:
			user = user[0]
			if request.POST.get('add', False):
				request_object = Request.objects.filter(user=user, content_type__pk=comic_type.id, object_id=comic.id, request_type=1)
				if not request_object:
					Request.objects.create(user=user, content_object=comic, request_type=1)
			elif request.POST.get('delete', False):
				if request.POST.get('assistant_type', False):
					request_object = Request.objects.get(user=user, content_type__pk=comic_type.id, object_id=comic.id, request_type=1)
					request_object.delete()
				else:
					comic.assistants.remove(user)
	requests = Request.objects.filter(content_type__pk=comic_type.id, object_id=comic.id, request_type=1)
	variables = {
		'comic': comic,
		'assistants': comic.assistants.all(),
		'assistants_requested':requests,
		'error': error
	}
	return render_to_response('comics/manage_assistants.html', variables, context_instance=RequestContext(request))

@login_required
def manage_links(request, id):
	
	comic = get_object_or_404(Comic, pk=id)
	if comic.user != request.user and request.user not in comic.assistants.all():
		return HttpResponseNotFound()
	form = LinkForm()
	if request.method == 'POST':
		if request.POST.get('delete', False):
			link = ComicLink.objects.get(pk=request.POST['link_id'])
			link.delete()
		else:
			print request.FILES
			form = LinkForm(request.POST, request.FILES)
			if form.is_valid():
				instance = form.save(commit=False)
				instance.comic = comic
				instance.save()
			form = LinkForm()
	variables = {
		'form': form,
		'comic':comic,
		'links':comic.links.order_by('type')
	}
	return render(request, 'comics/manage_links.html', variables)

@login_required
def edit_page(request, id):
	
	page = get_object_or_404(ComicPage, pk=id)
	if not page.comic.user == request.user and not request.user in page.comic.assistants.all() and not request.user.is_staff:
		return HttpResponseNotFound()
	if request.method == 'POST':
		form = PageForm(request.POST, request.FILES, instance=page)
		if form.is_valid():
			form.save()
			url = "/comics/pages/%s/" % page.comic.id
			return HttpResponseRedirect(url)
		else:
			print form.errors
	else:
		form = PageForm(instance=page)
	variables = {
		'comic': page.comic,
		'page': page,
		'form': form
	}
	return render_to_response('comics/edit_page.html', variables, context_instance=RequestContext(request))

@login_required
def add_page(request, id):
	
	comic = get_object_or_404(Comic, pk=id)
	if not comic.user == request.user and not request.user in comic.assistants.all() and not request.user.is_staff:
		return HttpResponseNotFound()
	form_count = int(request.GET.get('pages', 1))
	if form_count > 10:
	    form_count = 10
	PageFormSet = formset_factory(PageForm, extra=form_count, max_num=form_count)
	if request.method == 'POST':
#		form = PageForm(request.POST, request.FILES)
		formset = PageFormSet(request.POST, request.FILES)
#		if form.is_valid():
		if formset.is_valid():
			for form in formset:
				comic_page = form.save(commit=False)
				comic_page.comic = comic
				order = comic.pages.aggregate(Max('order_id')).get('order_id__max')
				if not order:
					order = 0
				comic_page.order_id = order + 1
				comic_page.save()
			if request.POST.get('another', False):
				formset = PageFormSet()
#				form = PageForm()
			else:
				path = "/comics/pages/%s/" % str(comic.id)
				return HttpResponseRedirect(path)
	else:
		formset = PageFormSet()
#		form = PageForm()
	variables = {
		'comic': comic,
#		'form': form
		'formset': formset,
		'form_count': form_count
	}
	return render(request, 'comics/add_page.html', variables)

@login_required
def create(request):
	
	form = ComicForm()
	if request.method == 'POST':
		form = ComicForm(request.POST, request.FILES)
		if form.is_valid():
			comic = form.save(commit=False)
			comic.user = request.user
			comic.seven_day_visits = 0
			comic.slug = comic.title.replace(' ', '_')
			comic.save()
			return HttpResponseRedirect('/comics/add_page/' + str(comic.id) + '/')
	variables = {
        'form': form
    }
	return render_to_response('comics/create.html', variables, context_instance=RequestContext(request))

@login_required
def edit(request, id):
	
	comic = get_object_or_404(Comic, pk=id)
	if not comic.user == request.user and not request.user in comic.assistants.all() and not request.user.is_staff:
		return HttpResponseNotFound()
	if request.method == 'POST':
		form = ComicEditForm(request.POST, request.FILES, instance=comic)
		if form.is_valid():
			form.save()
		else:
			print form.errors
	else:
		form = ComicEditForm(instance=comic)
	variables = {
		'form': form,
		'comic': comic
	}
	return render_to_response('comics/edit.html', variables, context_instance=RequestContext(request))

#@login_required
#def edit_layout(request, id):
#	
#	comic = get_object_or_404(Comic, pk=id)
#	if comic.user != request.user and request.user not in comic.assistants.all():
#		return HttpResponseNotFound()
#	if not comic.layout:
#		layout = Layout.objects.create(user=request.user, title=comic.title)
#		comic.layout = layout
#		comic.save()
#	layout = comic.layout
#	if request.method == 'POST':
#		form = LayoutForm(request.POST, request.FILES, instance=layout)
#		if form.is_valid():
#			form.save()
#			form = LayoutForm(instance=layout)
#		else:
#			print form.errors
#	else:
#		form = LayoutForm(instance=layout)
#	variables = {
#		'form': form,
#		'comic': comic,
#		'layout':layout,
#	}
#	return render_to_response('comics/edit_layout.html', variables, context_instance=RequestContext(request))


@login_required
def delete_comic(request, id):
    
    comic = get_object_or_404(Comic, pk=id)
    if not comic.user == request.user:
        return HttpResponseNotFound()
    comic.is_deleted = True
    comic.save()
    return HttpResponseRedirect('/user/' + request.user.username + '/#comics')

		
def check_title(request):
    
    response_str = "false"
    if request.is_ajax():
        title = request.GET.get('title').strip()
        if os.path.exists('/users/' + request.user.username + '/comics/' + title.replace(' ', '_')):
        	response_str = "true"
        if not Comic.objects.filter(title__iexact=title).exists():
            response_str = "true"
        return HttpResponse(response_str)

from layouts.forms import *

def comic_page(request, slug, page_id=None):
	
	try:
		comic = Comic.objects.get(slug__iexact=slug, is_deleted=False)
	except:
		return HttpResponseNotFound()
	if not can_see_comic(request.user, comic):
		return HttpResponseRedirect('/errors/age/')
	if comic.user == request.user or request.user in comic.assistants.all():
		pages = comic.pages.filter(is_deleted=False).order_by('order_id')
	else:
		pages = comic.pages.filter(is_deleted=False, post_date__lte=datetime.now()).order_by('order_id')
	page_count = pages.count()
	if page_count == 0:
		return HttpResponseRedirect('/errors/nopages/')
	
	# figure out this page and the pages around it for navigation
	current_page = None
	first_page = None
	last_page = None
	prev_page = None
	next_page = None
	pages_list = list(pages)
	if not page_id:
		if comic.enable_homepage:
			current_page = pages_list[0]
		else:
			current_page = pages_list[-1]
	else:
		current_page = get_object_or_404(ComicPage, pk=page_id, is_deleted=False)
	try:
		current_index = pages_list.index(current_page)
	except:
		return HttpResponseNotFound()
	if current_index != 0:
		first_page = pages_list[0]
		prev_page = pages_list[current_index - 1]
	if current_index != page_count - 1:
		last_page = pages_list[-1]
		next_page = pages_list[current_index + 1]

	# DUCKMARK STUFF
	duckmark_url = ''
	if request.user.is_authenticated():
		duckmark = comic.duckmarks.filter(user=request.user)
		if duckmark:
			duckmark_url = duckmark[0].url
    
    # get links
	header_links = ComicLink.objects.filter(comic=comic, type=0)
	other_comic_links = ComicLink.objects.filter(comic=comic, type=1)
	misc_links = ComicLink.objects.filter(comic=comic, type=2)
	
	variables = {
		'view':'comic',
        'comic': comic,
        'pages': pages,
        'header_links':header_links,
        'other_comic_links':other_comic_links,
        'misc_links':misc_links,
        'current_page': current_page,
        'first_page': first_page,
        'next_page': next_page,
        'prev_page': prev_page,
        'last_page': last_page,
        'duckmark_url':duckmark_url,
	}

	editmode = request.GET.get('editmode', False)
	if editmode:
		if comic.user == request.user or request.user in comic.assistants.all() or request.user.is_staff:
			editmode = True
			try:
				layout = comic.layout
			except:
				layout = Layout.objects.create(comic=comic)
			variables['editmode'] = True
			variables['layout_background_form'] = LayoutBackgroundForm(instance=comic.layout)
			variables['layout_page_form'] = LayoutPageForm(instance=comic.layout)
			variables['layout_author_form'] = LayoutAuthorForm(instance=comic.layout)
			variables['layout_nav_form'] = LayoutNavForm(instance=comic.layout)
			variables['layout_comment_form'] = LayoutCommentForm(instance=comic.layout)
	
	return render(request, 'comics/index.html', variables)

def featured(request):
    comics = FeaturedComic.objects.filter(comic__is_deleted=False, date_featured__lte=datetime.now()).order_by('-date_featured')
    if 'search' in request.GET:
        search = request.GET['search']
        if search != 'search featured archive':
            comics = comics.filter(comic__title__icontains=search) | comics.filter(comic__description__icontains=search) | comics.filter(comic__user__username__icontains=search)
    return render_to_response('comics/list_featured.html', {'comics': comics}, context_instance=RequestContext(request))

@login_required
def email_alerts(request, id):
    
    comic = Comic.objects.get(pk=id)
    if request.user in comic.email_alerts.all():
        comic.email_alerts.remove(request.user)
        return HttpResponse('You\'re no longer receiving email alerts from this comic.')
    else:
        comic.email_alerts.add(request.user)
        return HttpResponse('You\'re now receiving email alerts from this comic.\nNOTE: Please make sure to add updates@drunkduck.com to your safe sender list!')
        
@login_required
def add_recommend(request, id):
    
    comic = Comic.objects.get(pk=id)
    if request.user in comic.recommends.all():
        return HttpResponse('You already recommended this comic.')
    else:
        comic.recommends.add(request.user)
        return HttpResponse('You recommended this comic.')

@login_required
def remove_recommend(request, id):
    
    comic = Comic.objects.get(pk=id)
    comic.recommends.remove(request.user)
    return HttpResponseRedirect('/user/#recommends')

@login_required
def add_favorite(request, id):
    
    comic = Comic.objects.get(pk=id)
    if request.user in comic.favorites.all():
        return HttpResponse('You already favorited this comic.')
    else:
        comic.favorites.add(request.user)
        return HttpResponse('You favorited this comic.')

@login_required
def remove_favorite(request, id):
    
    comic = Comic.objects.get(pk=id)
    comic.favorites.remove(request.user)
    return HttpResponseRedirect('/user/favorites/')
