from django.contrib.auth.models import User
from django.db import models
from comics.models import Comic
from datetime import datetime

FORUM_CATEGORIES = (
    (1, 'general'),
    (2, 'community'),
    (3, 'project'),
    (6, 'other'),
)

#FORUM_CATEGORIES = (
#    (1, 'general'),
#    (2, 'community'),
#    (3, 'art/entertainment'),
#    (4, 'help'),
#    (5, 'feedback'),
#    (6, 'other'),
#)

class Signature(models.Model):
    
    user = models.OneToOneField(User, related_name='signature', null=True, blank=True)
    body = models.TextField()

class Forum(models.Model):
    
    user = models.ForeignKey(User, related_name='forums', null=True, blank=True)
    comic = models.ForeignKey(Comic, related_name='forums', null=True, blank=True)
    title = models.CharField(max_length=255)
    description = models.TextField(null=True, blank=True)    
    is_private = models.BooleanField(default=False)
    date_added = models.DateTimeField(auto_now_add=True)
    date_modified = models.DateTimeField(auto_now=True)
    order = models.IntegerField(default=1, blank=True)
    category = models.IntegerField(choices=FORUM_CATEGORIES, null=True, blank=True)
    moderators = models.ManyToManyField(User, related_name='forums_moderating', null=True, blank=True)
    is_deleted = models.BooleanField(default=False)
    old_id = models.IntegerField(null=True, blank=True)
    
    def __unicode__(self):
        
        return self.title
    
    class Meta:
        
        ordering = ('order',)

class Topic(models.Model):
    
    user = models.ForeignKey(User, related_name='topics', null=True, blank=True)
    forum = models.ForeignKey(Forum, related_name='topics')
    title = models.CharField(max_length=255)
    is_sticky = models.BooleanField(default=False)
    is_important = models.BooleanField(default=False)
    is_locked = models.BooleanField(default=False)    
    date_added = models.DateTimeField(auto_now_add=True)
    date_modified = models.DateTimeField(null=True)
    is_deleted = models.BooleanField(default=False)
    old_id = models.IntegerField(null=True, blank=True)
    
    def __unicode__(self):
        
        return self.title
    
    class Meta:
        
        ordering = ('-is_sticky', '-date_added',)

class Post(models.Model):
    
    user = models.ForeignKey(User, related_name='posts', null=True, blank=True)
    topic = models.ForeignKey(Topic, related_name='posts')
    body = models.TextField()
    date_added = models.DateTimeField(auto_now_add=True)
    date_modified = models.DateTimeField(auto_now=True)
    is_deleted = models.BooleanField(default=False)
    old_id = models.IntegerField(null=True, blank=True)
    
    def save(self, *args, **kwargs):
        
        id = self.id
        super(Post, self).save(*args, **kwargs)
        if not id:
            self.topic.date_modified = datetime.now()
            self.topic.save()
    
    def __unicode__(self):
        
        return self.user.username + ' - ' + str(self.date_added)
    
    class Meta:
        
        ordering = ('date_added',)