from django.contrib.auth.decorators import login_required
from django.http import HttpResponseRedirect, HttpResponseNotFound
from django.shortcuts import render_to_response, get_object_or_404, render
from django.template import RequestContext
from django.utils.html import linebreaks
from forums.models import *
from forums.forms import *
from comics.models import *
from forums.templatetags.forum_tags import stripquotes

@login_required
def edit_forum(request, id):
    
    forum = get_object_or_404(Forum, pk=id, is_deleted=False)
    url = "/forum/"
    comic = None
    if forum.comic:
        if forum.comic.user != request.user and request.user not in forum.comic.assistants.all() and not request.user.is_staff:
            return HttpResponseNotFound()
        url = "/forum/comic/%s/" % forum.comic.id
        comic = forum.comic
    elif request.user != forum.user and not request.user.is_staff:
        return HttpResponseNotFound()
    if request.method == 'POST':
        form = ForumForm(request.POST, instance=forum)
        if form.is_valid():
            form.save()
            return HttpResponseRedirect(url)
    else:
        form = ForumForm(instance=forum)
    variables = {
        'form':form,
        'comic':comic
    }
    return render(request, 'forums/add_forum.html', variables)

@login_required
def delete_forum(request, id):
    
    forum = get_object_or_404(Forum, pk=id, is_deleted=False)
    if forum.comic:
        if forum.comic.user != request.user and request.user not in forum.comic.assistants.all() and not request.user.is_staff:
            return HttpResponseNotFound()
    elif request.user != forum.user and not request.user.is_staff:
        return HttpResponseNotFound()
    forum.is_deleted = True
    forum.save()
    return HttpResponseRedirect(request.META['HTTP_REFERER'])

@login_required
def add_forum(request):
    
    if not request.user.is_staff:
        return HttpResponseNotFound()
    if request.method == 'POST':
        form = ForumForm(request.POST)
        if form.is_valid():
            instance = form.save(commit=False)
            instance.user = request.user
            instance.save()
            return HttpResponseRedirect('/forum/')
        else:
            print form.errors
    else:
        form = ForumForm()
    variables = {
        'form':form
    }
    return render(request, 'forums/add_forum.html', variables)

@login_required
def add_comic_forum(request, id):
    
    comic = get_object_or_404(Comic, pk=id)
    if comic.user != request.user and request.user not in comic.assistants.all():
        return HttpResponseNotFound()
    if request.method == 'POST':
        form = ForumForm(request.POST)
        if form.is_valid():
            instance = form.save(commit=False)
            instance.user = request.user
            instance.comic = comic
            instance.save()
            return HttpResponseRedirect('/forum/comic/' + str(comic.id) + '/')
        else:
            print form.errors
    else:
        form = ForumForm()
    variables = {
        'comic':comic,
        'form':form
    }
    return render(request, 'forums/add_forum.html', variables)

def list_comic_forums(request, id):
    
    comic = get_object_or_404(Comic, pk=id)
    forums = Forum.objects.filter(comic=comic, is_deleted=False).order_by('order')
    variables = {
        'comic':comic,
        'forums':forums
    }
    return render(request, 'forums/list_comic_forums.html', variables)    

def list_forums(request):
    
    if request.user.is_staff:
        forums = Forum.objects.filter(is_deleted=False, comic__isnull=True).order_by('category', 'order')
    else:
        forums = Forum.objects.filter(is_deleted=False, comic__isnull=True, is_private=False).order_by('category', 'order')
    return render(request, 'forums/list_forums.html', {'forums':forums})    

def list_topics(request, id):
    
    forum = get_object_or_404(Forum, id=id, is_deleted=False)
    if forum.is_private and not request.user.is_staff:
        return HttpResponseNotFound()
    if 'search' in request.GET:
        topics = forum.topics.filter(title__icontains=request.GET['search'], is_deleted=False).order_by('-is_sticky', '-date_modified')
    else:
        topics = forum.topics.filter(is_deleted=False).order_by('-is_sticky', '-date_modified')
    # add permissions for if they are staff
    return render_to_response('forums/list_topics.html', {'forum':forum, 'topics':topics}, context_instance=RequestContext(request))

def list_posts(request, id):
    
    topic = get_object_or_404(Topic, id=id, is_deleted=False)
    if topic.forum.is_private and not request.user.is_staff:
        return HttpResponseNotFound()
    if 'search' in request.GET:
        posts = topic.posts.filter(body__icontains=request.GET['search'], is_deleted=False).order_by('date_added')
    else:
        posts = topic.posts.filter(is_deleted=False).order_by('date_added')
    # add permissions for if they are staff
    return render_to_response('forums/list_posts.html', {'topic':topic, 'posts':posts}, context_instance=RequestContext(request))

@login_required
def add_post(request, id):
    
    topic = Topic.objects.get(pk=id, is_deleted=False)
    if topic.forum.is_private and not request.user.is_staff:
        return HttpResponseNotFound()
    if topic.is_locked:
        return HttpResponseNotFound()
    form = PostForm()
    count = topic.posts.count()
    post_number = count % 30
    if post_number == 0:
        post_number = 30
    posts = topic.posts.filter(is_deleted=False)[count-post_number:]
                
    if request.method == "POST":
        if 'add' in request.POST:
            form = PostForm(request.POST)
            if form.is_valid():
                post = form.save(commit=False)
                post.topic = topic
                post.user = request.user
                post.save()
                count = topic.posts.count()
                page_number = count / 30
                if count <= 30:
                    page_number = 1
                elif count % 30:
                    page_number += 1
                return HttpResponseRedirect("/forum/topic/%s/?page=%s#latest" % (str(topic.id), str(page_number)))
    else:
        try:
            quoted_post = Post.objects.get(pk=request.GET['quote'])
            form.initial['body'] = "<br /><blockquote><div class=\"author\"><a class=\"blue\" href=\"http://www.drunkduck.com/user/%s/\">%s</a> wrote:</div>%s</blockquote><br />" % (quoted_post.user.username, quoted_post.user.username, linebreaks(stripquotes(quoted_post.body)))
        except:
            pass
    variables = {
        'form': form,
        'posts': posts,
        'topic': topic
    }
    return render(request, 'forums/add_post.html', variables)

@login_required
def edit_post(request, id):
    
    post = Post.objects.get(id=id, is_deleted=False)
    if post.user != request.user and not request.user.is_staff:
        return HttpResponseNotFound()
    count = post.topic.posts.count()
    
    post_list = list(post.topic.posts.filter(is_deleted=False))
    post_index = post_list.index(post)
    
    page_number = post_index / 30
    if post_index <= 30:
        page_number = 1
    elif post_index % 30:
        page_number += 1
            
    form = PostForm(instance=post)
    if request.method == "POST":
        form = PostForm(request.POST, instance=post)
        if form.is_valid():
            form.save()
            return HttpResponseRedirect("/forum/topic/%s/?page=%s#%s" % (str(post.topic.id), str(page_number), str(post.id)))
    else:
        form.initial['body'] = linebreaks(form.initial['body'])
    return render_to_response('forums/add_post.html', {'form':form, 'topic':post.topic}, context_instance=RequestContext(request))


@login_required
def add_topic(request, id):
    
    forum = get_object_or_404(Forum, pk=id, is_deleted=False)
    if forum.is_private and not request.user.is_staff:
        return HttpResponseNotFound()
    form = TopicForm()
    post_form = PostForm()
    if request.method == "POST":
        form = TopicForm(request.POST)
        post_form = PostForm(request.POST)
        if form.is_valid() and post_form.is_valid():
            topic = form.save(commit=False)
            topic.forum = forum
            topic.user = request.user
            topic.save()
            post = post_form.save(commit=False)
            post.topic = topic
            post.user = request.user
            post.save()
            return HttpResponseRedirect('/forum/' + str(forum.id) + '/')
        else:
            print 'post', form.errors
            print 'topic', form.errors
    return render_to_response('forums/add_topic.html', {'form':form, 'post_form':post_form, 'forum':forum}, context_instance=RequestContext(request))

@login_required
def edit_topic(request, id):
    
    topic = get_object_or_404(Topic, pk=id, is_deleted=False)
    if topic.user != request.user and not request.user.is_staff and topic.forum.user != request.user:
        return HttpResponseNotFound()
    form = TopicForm(instance=topic)
    if request.method == "POST":
        form = TopicForm(request.POST, instance=topic)
        if form.is_valid():
            form.save()
            return HttpResponseRedirect('/forum/topic/' + str(topic.id) + '/')
    return render_to_response('forums/edit_topic.html', {'form':form, 'topic':topic}, context_instance=RequestContext(request))

@login_required
def delete_topic(request, id):
    
    topic = get_object_or_404(Topic, pk=id, is_deleted=False)
    if topic.user == request.user or request.user.is_staff or topic.forum.user == request.user:
        topic.is_deleted = True
        topic.save()
    return HttpResponseRedirect('/forum/' + str(topic.forum.id) + '/')

@login_required
def delete_post(request, id):
    
    post = get_object_or_404(Post, pk=id)
    if post.user == request.user or request.user.is_staff or post.topic.forum.user == request.user or post.topic.user == request.user:
        post.is_deleted = True
        post.save()
    return HttpResponseRedirect('/forum/topic/' + str(post.topic.id) + '/')
