from django.shortcuts import render_to_response, get_object_or_404, render
from django.http import HttpResponse, HttpResponseRedirect, HttpResponseNotFound
from django.contrib.auth.decorators import login_required
from django.template import RequestContext

from imagelocker.forms import ImageForm
from imagelocker.models import ImageFile


@login_required
def delete(request, id):
    image = get_object_or_404(Image, id=id)
    image.delete()
    return HttpResponseRedirect('/attachments/list/')


@login_required
def upload_image(request):
    if request.method == 'POST':
        form = ImageForm(request.POST, request.FILES)
        if form.is_valid():
            image = form.save(commit=False)
            image.user = request.user
            image.filename = str(image.image).split('/')[-1]
            image.save()


@login_required
def display(request, filename):
    image = get_object_or_404(Image, filename=filename)
    return HttpResponseRedirect(image.image.url)

@login_required
def list_images(request):
    if request.method == 'POST':
        form = ImageForm(request.POST, request.FILES)
        if form.is_valid():
            image_obj = form.save(commit=False)
            image_obj.user = request.user
            image_obj.save()
        else:
            print form.errors
    else:
        form = ImageForm()
    images = request.user.imagefiles.all()
    variables = {
        'images': images,
        'form':form,
    }
    return render(request, 'imagelocker/list_images.html', variables)

@login_required
def delete_image(request, image_id):
    if request.method == 'GET':
        image_obj = get_object_or_404(ImageFile, pk=image_id)

        if not image_obj.user == request.user:
            return HttpResponseNotFound()

        image_obj.delete()
        return HttpResponseRedirect('/imagelocker/')
