from django.core.management import setup_environ
import settings
setup_environ(settings)

import os
from boto.s3.connection import S3Connection
from boto.s3.key import Key
from cStringIO import StringIO
from PIL import Image, ImageOps
from urllib import urlopen

IMAGE_FORMATS = ('png', 'jpg', 'jpeg', 'gif')

def load_image_from_url(imagepath):

    StringIO().close()
    file = urlopen(imagepath)
    io = StringIO(file.read())
    return Image.open(io)

def rescale(im, width, height=None, force=False):
    
    if not height:
        height = im.size[1]
    if not force:
        if im.size[0] <= width and im.size[1] <= height:
            return im, False
        im.thumbnail((width, height), Image.ANTIALIAS)
    else:
        if im.size[0] == width and im.size[1] == height:
            return im, False
        im = ImageOps.fit(im, (width, height), method=Image.ANTIALIAS, centering=(0.5, 0.5))
    return im, True
    
def upload_image(image, path, format='png'):
    
    format = format.lower()
    if format == 'jpg':
        format = 'jpeg'
    io = StringIO() 
    local_path = 'media/' + path
    dir = local_path.split('/')
    dir.pop()
    dir = '/'.join(dir)
    if not os.path.exists(dir):
        os.makedirs(dir)
    image.save('media/' + path, format=format, quality=100)
    image.save(io, format=format, quality=100)   
    conn = S3Connection(settings.AWS_ACCESS_KEY, settings.AWS_SECRET_KEY)
    b = conn.get_bucket("media.drunkduck.com")
    k = Key(b)
    k.key = path
    k.set_metadata("Content-Type", 'image/' + format)
    k.set_contents_from_file(io)
    k.set_acl("public-read")
    io.close()
    del image