"""
#########################################################################
TUPLES
#########################################################################
"""

BACKGROUND_REPEAT_CHOICES = (
    (0, 'repeat'),
    (1, 'no-repeat'),
    (2, 'repeat-x'),
    (3, 'repeat-y'),
)

BACKGROUND_POSITION_CHOICES = (
    (0, 'left top'),
    (1, 'left center'),
    (2, 'left bottom'),
    (3, 'right top'),
    (4, 'right center'),
    (5, 'right bottom'),
    (6, 'center top'),
    (7, 'center center'),
    (8, 'center bottom'),
)

LINK_DECORATION_CHOICES = (
    (0, 'none'),
    (1, 'underline'),
    (2, 'line-through'),
)

NAV_POSITION_CHOICES = (
    (0, 'both'),
    (1, 'top'),
    (2, 'bottom'),
)

FONT_SIZE_CHOICES = (
    (10, '10'),
    (12, '12'),
    (14, '14'),
    (16, '16'),
)

PAGE_BORDER_SIZES = (
    (0, '0'),
    (5, '5'),
    (10, '10'),
    (15, '15'),
    (20, '20'),
)

"""
#########################################################################
MODELS
#########################################################################
"""

from django.db import models
from comics.models import Comic

class Layout(models.Model):
    
    comic = models.OneToOneField(Comic, related_name='layout')
    
    background_image = models.URLField('background image', blank=True, null=True)
    background_color = models.CharField('background color', max_length=6, null=True, blank=True)
    background_repeat = models.IntegerField('background repeat', choices=BACKGROUND_REPEAT_CHOICES, default=0)
    background_position = models.IntegerField('background position', choices=BACKGROUND_POSITION_CHOICES, default=0)
    background_fixed = models.BooleanField('background fixed?', default=False)
    
    page_border_color = models.CharField('border color', max_length=6, null=True, blank=True)
    page_border_size = models.IntegerField('border size', choices=PAGE_BORDER_SIZES, default=0)
    header_color = models.CharField('header color', max_length=6, null=True, blank=True)
    header_bold = models.BooleanField(default=False)
    header_italic = models.BooleanField(default=False)
    text_color = models.CharField(max_length=6, null=True, blank=True)
    text_size = models.IntegerField(choices=FONT_SIZE_CHOICES, default=12)
    text_bold = models.BooleanField(default=False)
    text_italic = models.BooleanField(default=False)
    link_color = models.CharField('link color', max_length=6, blank=True, null=True)
    link_decoration = models.IntegerField(choices=LINK_DECORATION_CHOICES, default=0)
    link_hover_color = models.CharField(max_length=6, blank=True, null=True)
    link_hover_decoration = models.IntegerField(choices=LINK_DECORATION_CHOICES, default=0)
    
    author_background_color = models.CharField(max_length=6, null=True, blank=True)
    author_header_color = models.CharField(max_length=6, null=True, blank=True)
    author_header_bold = models.BooleanField(default=False)
    author_header_italic = models.BooleanField(default=False)
    author_text_color = models.CharField(max_length=6, null=True, blank=True)
    author_text_size = models.IntegerField(choices=FONT_SIZE_CHOICES, default=12)
    author_text_bold = models.BooleanField(default=False)
    author_text_italic = models.BooleanField(default=False)
    author_link_color = models.CharField('link color', max_length=6, blank=True, null=True)
    author_link_decoration = models.IntegerField(choices=LINK_DECORATION_CHOICES, default=0)
    author_link_hover_color = models.CharField(max_length=6, blank=True, null=True)
    author_link_hover_decoration = models.IntegerField(choices=LINK_DECORATION_CHOICES, default=0)
    
    comment_background_color = models.CharField(max_length=6, null=True, blank=True)
    comment_header_color = models.CharField(max_length=6, null=True, blank=True)
    comment_header_bold = models.BooleanField(default=False)
    comment_header_italic = models.BooleanField(default=False)
    comment_text_color = models.CharField(max_length=6, null=True, blank=True)
    comment_text_size = models.IntegerField(choices=FONT_SIZE_CHOICES, default=12)
    comment_text_bold = models.BooleanField(default=False)
    comment_text_italic = models.BooleanField(default=False)
    comment_link_color = models.CharField('link color', max_length=6, blank=True, null=True)
    comment_link_decoration = models.IntegerField(choices=LINK_DECORATION_CHOICES, default=0)
    comment_link_hover_color = models.CharField(max_length=6, blank=True, null=True)
    comment_link_hover_decoration = models.IntegerField(choices=LINK_DECORATION_CHOICES, default=0)
    
    arrow_first = models.URLField(blank=True, null=True)
    arrow_prev = models.URLField(blank=True, null=True)
    arrow_next = models.URLField(blank=True, null=True)
    arrow_last = models.URLField(blank=True, null=True)
    nav_position = models.IntegerField(choices=NAV_POSITION_CHOICES, default=0)
    
    def __unicode__(self):
        
        return self.comic.title