from django.contrib.auth.models import User
from django.db import models

class Message(models.Model):
    
    sender = models.ForeignKey(User, related_name="outbox")
    read = models.BooleanField(default=False, blank=True)
    sender_deleted = models.BooleanField(default=False, blank=True)
    recipient = models.ForeignKey(User, null=True, blank=True, related_name='inbox')
    recipient_deleted = models.BooleanField(default=False, blank=True)
    subject = models.TextField(null=True, blank=True)
    body = models.TextField()
    date_sent = models.DateTimeField(auto_now_add=True, blank=True)
    parent = models.ForeignKey('self', related_name='children', null=True, blank=True)
    
    def check_permissions(self, user):
        
        if self.sender == user or self.recipient == user:
            return True
        return False            
    
    def add_reply(self, user, data):
        
        parent_message = self.parent
        if self.parent.sender == user:
            recipient = self.parent.recipient
        elif self.parent.recipient == user:
            recipient = self.parent.sender
        Message.objects.create(sender=user, recipient=recipient, subject=self.parent.subject, body=data['body'], parent=self.parent)
    
    def toggle_delete(self, user, type):
        
        messages = self.parent.children.all()
        for message in messages:
            if type == 'inbox' and message.recipient == user:
                message.recipient_deleted = True
            elif type == 'sent' and message.sender == user:
                message.sender_deleted = True
            message.save()
    
    def toggle_read(self, user):
    
        messages = self.parent.children.all()
        for message in messages:
            if message.recipient == user:
                message.read = True
                message.save()
            
    def __unicode__(self):
        
        return self.sender.username + ' - ' + str(self.date_sent)
    
    class Meta:
        
        ordering = ('-date_sent',)