from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.http import HttpResponseRedirect, HttpResponseNotFound, HttpResponse
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from messages.models import *
from messages.forms import *

@login_required
def inbox(request):
    
    if request.method == 'POST':
        if 'delete' in request.POST:
            message_id_list = request.POST.getlist('messages')
            messages = Message.objects.filter(pk__in=message_id_list)
            messages.update(recipient_deleted=True)
    existing_threads = []
    all_messages = Message.objects.filter(recipient=request.user, recipient_deleted=False).order_by('-date_sent')
    messages = []
    for message in all_messages:
        if not message.parent_id in existing_threads:
            existing_threads.append(message.parent_id)
            messages.append(message)
    return render_to_response('messages/mailbox.html', {'messages':messages, 'sub_view': 'inbox'}, context_instance=RequestContext(request)) 

@login_required
def sent(request):
    
    if request.method == 'POST':
        if 'delete' in request.POST:
            message_id_list = request.POST.getlist('messages')
            messages = Message.objects.filter(pk__in=message_id_list)
            messages.update(sender_deleted=True)
    existing_threads = []
    all_messages = Message.objects.filter(sender=request.user, sender_deleted=False).order_by('-date_sent')
    messages = []
    for message in all_messages:
        if not message.parent_id in existing_threads:
            existing_threads.append(message.parent_id)
            messages.append(message)
    return render_to_response('messages/mailbox.html', {'messages':messages, 'sub_view': 'sent'}, context_instance=RequestContext(request))    


@login_required
def compose(request):
    
    form = MessageForm()
    if 'user' in request.GET:
        form.initial['to'] = request.GET['user']
    if request.method == 'POST':
        form = MessageForm(request.POST)
        if form.is_valid():
            recipient = User.objects.get(username=form.cleaned_data['to'])
            message = Message.objects.create(sender=request.user, 
                                   recipient=recipient, 
                                   subject=form.cleaned_data['subject'],
                                   body=form.cleaned_data['body'])
            message.parent_id = message.id
            message.save()
            return render_to_response('messages/compose_complete.html', {'sub_view': 'compose'}, context_instance=RequestContext(request))
        else:
            print form.errors
    return render_to_response('messages/compose.html', {'form':form, 'sub_view': 'compose'}, context_instance=RequestContext(request))

@login_required
def display(request, id):
    
    message = get_object_or_404(Message, id=id)
    reply_form = ReplyForm()
    if not message.check_permissions(request.user):
        return HttpResponseNotFound()
    if request.method == 'POST':
        if 'reply' in request.POST:
            reply_form = ReplyForm(request.POST)
            if reply_form.is_valid():
                message.add_reply(request.user, reply_form.cleaned_data)
                reply_form = ReplyForm()
        if 'delete' in request.POST:
            message.toggle_delete(request.user, request.GET['t'])
            return HttpResponseRedirect('/quack/' + request.GET['t'])
    if request.GET['t'] == 'inbox':
        message.toggle_read(request.user)
    replies = Message.objects.filter(parent=message.parent).order_by('date_sent')
    return render_to_response('messages/display.html', {'subject': message.subject, 'replies':replies, 'reply_form':reply_form}, context_instance=RequestContext(request))
