from django.db import models
from django.contrib.auth.models import User
from tagging.fields import TagField
from tagging.models import Tag
from ratings.models import Like
#from tagging_autocomplete.models import TagAutocompleteField
from comments.models import Comment
from django.db.models.signals import post_save, pre_save
from django.template.defaultfilters import slugify
from datetime import datetime

class Article(models.Model):
    title = models.TextField()
    is_live = models.BooleanField(default=False)
    urgent = models.BooleanField('Urgent News', default=False)
    body = models.TextField()
    date_live = models.DateTimeField(default=datetime.now())
    date_added = models.DateTimeField(auto_now_add=True)
    date_modified = models.DateTimeField(auto_now=True)
    author = models.ForeignKey(User, related_name='articles', null=True, blank=True)
    tags = TagField()
    slug = models.SlugField(
        unique_for_date='date_added',
        help_text='Automatically built from the title.',
        blank=True,
        null=True,
	max_length=750
    )
    likes = models.ManyToManyField(Like, null=True, blank=True)
    comments = models.ManyToManyField(Comment, null=True, blank=True)
    
    class Meta:
        ordering = ('-date_added',)
        get_latest_by = 'date_added'
        verbose_name_plural = 'News Articles'

    def __unicode__(self):
        return u'%s' %(self.title)

    def get_absolute_url(self):
        return "/news/%s/%s/" %(self.date_added.strftime("%Y/%b/%d").lower(), self.slug)
    
    def get_previous_published(self):
        return self.get_previous_by_date_added(status__exact=1)

    def get_next_published(self):
        return self.get_next_by_date_added(status__exact=1)

    def get_tags(self):
        return Tag.objects.get_for_object(self)


def generate_slug(sender, **kwargs):
    kwargs['instance'].slug = slugify(kwargs['instance'].title)


pre_save.connect(generate_slug, sender=Article, dispatch_uid='slug_updater')
