from django.db import models
from django.contrib.auth.models import User
#from django.template.defaultfilters import slugify
from comments.models import Comment
from ratings.models import Like
from tagging.fields import TagField
from datetime import datetime

class LiveManager(models.Manager):
    
    def get_query_set(self):
    
        return super(LiveManager, self).get_query_set().filter(is_live=True, date_live__lte=datetime.now())

class Episode(models.Model):

    user = models.ForeignKey(User, null=True, blank=True)
    is_live = models.BooleanField(default=True)
    title = models.CharField(max_length=200)
    url = models.URLField(blank=True, null=True, max_length=150)
    date_live = models.DateTimeField(default=datetime.now())
    summary = models.TextField(blank=True, null=True)
    notes = models.TextField(blank=True, null=True)
    date_added = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    slug = models.SlugField(
        help_text='Automatically built from the title. -- DO NOT MODIFY',
        blank=True,
        null=True, max_length=150
    )
    tags = TagField()
    comments = models.ManyToManyField(Comment, null=True, blank=True)
    likes = models.ManyToManyField(Like, null=True, blank=True)

    objects = models.Manager()  # Declare this manager first to use default manager for admin
    live = LiveManager()

    def __unicode__(self):
        
        return self.title

    class Meta:

        ordering = ('-date_added',)

#    def save(self):

#        self.slug = slugify(self.title[:50])
#        super(Episode, self).save()