from django.db import models
from django.contrib.auth.models import User

RATING_RANGE = (
    (1, '*'),
    (2, '**'),
    (3, '***'),
    (4, '****'),
    (5, '*****')
)

LIKE_CHOICES = (
    (0, 'like'),
    (1, 'dislike'),
)

class Rating(models.Model):
    
    user = models.ForeignKey(User)
    rating = models.IntegerField(choices=RATING_RANGE)
    date_rated = models.DateTimeField(auto_now_add=True)
    date_modified = models.DateTimeField(auto_now=True)

    def __unicode__(self):
        return self.user.username + ': ' + str(self.rating)

def add_rating(user, object, rating):
    
    rating_object = object.ratings.filter(user=user)
    if rating_object:
        rating_object.update(rating=rating)
    else:
        rating_object = Rating.objects.create(user=user, rating=rating)
        object.ratings.add(rating_object)
    return rating_object

class Like(models.Model):
    
    user = models.ForeignKey(User)
    like_type = models.IntegerField(choices=LIKE_CHOICES, default=0)
    date_liked = models.DateTimeField(auto_now_add=True)
    date_modified = models.DateTimeField(auto_now=True)
    
def add_like(user, object, like_type=0):
    
    like = object.likes.filter(user=user)
    if like:
        if like.like_type != like_type:
            like.update(like_type=like_type)
    else:
        like = Like.objects.create(user=user, like_type=like_type)
        object.likes.add(like)
    return like