from django.db import models
from django.contrib.auth.models import User
from tagging.fields import TagField
from tagging.models import Tag
#from tagging_autocomplete.models import TagAutocompleteField
#from django.db.models.signals import post_save, pre_save
from django.template.defaultfilters import slugify

ART_STATUS = (
    ('Approved', 'Approved'),
    ('Rejected', 'Rejected'),
    ('Deleted', 'Deleted'),
    ('Pending Approval', 'Pending Approval'),
)
ART_RATING = (
    ('Mature', 'Mature'),
    ('Family Friendly', 'Family Friendly'),
)

class Artworkcategory(models.Model):
    title = models.CharField(max_length=200)
    disporder = models.IntegerField(max_length=4)
    class Meta:
        ordering = ('disporder')
        verbose_name_plural = 'Artwork Categories'
        verbose_name = 'Artwork Category'
    def __unicode__(self):
        return u'%s' %(self.title)

class Artwork(models.Model):
    status = models.IntegerField('Publish Status',choices=ART_STATUS, default=0)
    rating = models.IntegerField('Publish Status',choices=ART_RATING, default=0)
    isactive = models.BooleanField('Is Artwork Active?',default=False, blank=True)
    ismature = models.BooleanField('Is Mature Content?',default=False, blank=True)
    isfeatured = models.BooleanField('Is a Featured Product?',default=False, blank=True)
    isbestseller = models.BooleanField('Is a Best Seller',default=False, blank=True)
    ispromo = models.BooleanField('Is a Promo Product?',default=False, blank=True)
    sku = models.ForeignKey(User, related_name='SKU', null=True, blank=True)
    title = models.CharField(max_length=200)
    desc_html = models.TextField(blank=True,null=True)
    desc_markdown = models.TextField('Content',blank=True,null=True)
    upload_date = models.DateTimeField(auto_now_add=True, blank=True)
    lastupdate_date = models.DateTimeField(blank=True)
    thumbnail = models.ImageField(upload_to="uploads", null=True, blank=True)
    tags = TagField()
    slug = models.SlugField(
        unique_for_date='date_added',
        help_text='Automatically built from the title.',
        blank=True,
        null=True
    )
    comments = models.ManyToManyField(Comment, null=True, blank=True)
    
    class Meta:
        ordering = ('-date_added',)
        get_latest_by = 'date_added'
        verbose_name_plural = 'News Articles'

    def __unicode__(self):
        return u'%s' %(self.title)

    def get_absolute_url(self):
        return "/news/%s/%s/" %(self.date_added.strftime("%Y/%b/%d").lower(), self.slug)
    
    def get_previous_published(self):
        return self.get_previous_by_date_added(status__exact=1)

    def get_next_published(self):
        return self.get_next_by_date_added(status__exact=1)

    def get_tags(self):
        return Tag.objects.get_for_object(self)


def generate_slug(sender, **kwargs):
    kwargs['instance'].slug = slugify(kwargs['instance'].title)


pre_save.connect(generate_slug, sender=Article, dispatch_uid='slug_updater')
