from django.db import models

"""
#########################################################################
MANAGERS
#########################################################################
"""

class LiveSubPageManager(models.Manager):
    
    def get_query_set(self):
    
        return super(LiveSubPageManager, self).get_query_set().filter(is_deleted=False)   

"""
#########################################################################
MODELS
#########################################################################
"""

from comics.models import Comic
from django.template.defaultfilters import slugify

class SubPage(models.Model):
    
    comic = models.ForeignKey(Comic, related_name='subpages')
    is_deleted = models.BooleanField(default=False)
    title = models.CharField(max_length=255)
    description = models.TextField(null=True, blank=True)
    content = models.TextField()
    slug = models.SlugField(max_length=255)
    date_created = models.DateTimeField(auto_now_add=True)
    date_modified = models.DateTimeField(auto_now=True)
    
    objects = models.Manager()  # Declare this manager first to use default manager for admin
    live = LiveSubPageManager()
                
    def save(self, *args, **kwargs):
        
        self.slug = slugify(self.title)    
        super(SubPage, self).save(*args, **kwargs)
            
    def __unicode__(self):
        
        return "%s - %s" % (self.comic.title, self.title)
    
    class Meta:
        
        verbose_name = 'Sub Page'
        verbose_name_plural = 'Sub Pages'