from django.contrib.auth.decorators import login_required
from django.http import HttpResponseRedirect, HttpResponseNotFound
from django.shortcuts import render
from comics.models import Comic
from subpages.models import SubPage
from subpages.forms import SubPageForm

@login_required
def delete_subpage(request, subpage_id):
    
    try:
        subpage = SubPage.live.get(pk=subpage_id)
    except:
        return HttpResonseNotFound()
    if not subpage.comic.user == request.user:
        return HttpResonseNotFound()
    subpage.is_deleted = True
    subpage.save()
    url = "/subpages/list/%s/" % subpage.comic.id
    return HttpResponseRedirect(url)

@login_required
def add_subpage(request, comic_id):
    
    try:
        comic = Comic.live.get(pk=comic_id)
    except:
        return HttpResponseNotFound()
    if request.method == "POST":
        form = SubPageForm(request.POST)
        if form.is_valid():
            instance = form.save(commit=False)
            instance.comic = comic
            instance.save()
            url = "/subpages/list/%s/" % comic.id
            return HttpResponseRedirect(url)
    else:
        form = SubPageForm()
    variables = {'form':form, 'comic':comic,}
    return render(request, 'subpages/add.html', variables)

@login_required
def edit_subpage(request, subpage_id):
    
    try:
        subpage = SubPage.live.get(pk=subpage_id)
    except:
        return HttpResponseNotFound()
    if request.method == "POST":
        form = SubPageForm(request.POST, instance=subpage)
        if form.is_valid():
            form.save()
    else:
        form = SubPageForm(instance=subpage)
    variables = {'form':form, 'subpage':subpage}
    return render(request, 'subpages/edit.html', variables)

@login_required
def list_subpage(request, comic_id):
    
    try:
        comic = Comic.live.get(pk=comic_id)
    except:
        return HttpResponseNotFound()
    subpages = SubPage.live.filter(comic=comic)
    variables = {'subpages':subpages, 'comic':comic,}
    return render(request, 'subpages/list.html', variables)

@login_required
def display_subpage(request, comic_slug, subpage_slug):
    
    try:
        comic = Comic.live.get(slug=comic_slug)
        subpage = SubPage.live.get(comic=comic, slug=subpage_slug)
    except:
        return HttpResponseNotFound()
    variables = {
        'comic':comic,
        'subpage':subpage,
    }
    return render(request, 'subpages/display.html', variables)
        