from django.contrib.auth.decorators import login_required
from django.db.models import Avg
from django.http import HttpResponseRedirect, HttpResponseNotFound
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from tutorials.forms import *
from tutorials.models import *

def list(request):
    
    featured_tutorials = FeaturedTutorial.objects.order_by('?')[:2]
    tutorials = Tutorial.objects.filter(is_deleted=False)
    if 'search' in request.GET:
        search = request.GET['search'].strip()
        if search != 'search tutorials':
            tutorials = tutorials.filter(title__icontains=search) | tutorials.filter(description__icontains=search) | tutorials.filter(tags__icontains=search)
    sort = ''
    if 'sort' in request.GET:
        if request.GET['sort']:
            sort = request.GET['sort']
            if sort == 'title':
                tutorials = tutorials.order_by('title')
            elif sort == 'date':
                tutorials = tutorials.order_by('-date_added')
            elif sort == 'rating':
                tutorials = tutorials.annotate(avg=Avg('ratings__rating')).order_by('-avg')
    else:
        tutorials = tutorials.order_by('-id')
    return render_to_response('tutorials/list.html', {'tutorials':tutorials, 'featured_tutorials':featured_tutorials, 'sort':sort}, context_instance=RequestContext(request))

def display(request, id):
    
    tutorial = get_object_or_404(Tutorial, id=id, is_deleted=False)
    return render_to_response('tutorials/display.html', {'tutorial':tutorial}, context_instance=RequestContext(request))

@login_required
def add(request):
    
    if  request.method == 'POST':
        form = TutorialForm(request.POST)
        if form.is_valid():
            print 'valid'
            tutorial = form.save(commit=False)
            tutorial.user = request.user
            tutorial.save()
            return HttpResponseRedirect('/tutorial/')
    else:
        form = TutorialForm()
    return render_to_response('tutorials/add.html', {'form':form}, context_instance=RequestContext(request))

@login_required
def edit(request, id):
    
    tutorial = get_object_or_404(Tutorial, id=id)
    if tutorial.user != request.user:
        return HttpResponseNotFound()
    if  request.method == 'POST':
        form = TutorialForm(request.POST, instance=tutorial)
        if form.is_valid():
            form.save()
            return HttpResponseRedirect('/tutorial/' + str(tutorial.id) + '/')
    else:
        form = TutorialForm(instance=tutorial)
    return render_to_response('tutorials/add.html', {'form':form}, context_instance=RequestContext(request))

@login_required
def delete(request, id):
    
    tutorial = get_object_or_404(Tutorial, pk=id)
    if tutorial.user == request.user:
        tutorial.is_deleted = True
        tutorial.save()
    return HttpResponseRedirect('/tutorial/')