from django.contrib import admin
from users.models import *
import datetime


#
# ACTIONS
#

def ban_users(modeladmin, request, queryset):
    queryset.update(banned=True)


def unban_users(modeladmin, request, queryset):
    queryset.update(banned=False)


def freeze_users(modeladmin, request, queryset):
    queryset.update(frozen=True)


def unfreeze_users(modeladmin, request, queryset):
    queryset.update(frozen=False)


def deactivate_users(modeladmin, request, queryset):
    users = [item.user for item in queryset]
    for user in users:
        user.is_active = False
        user.save()


def reactivate_users(modeladmin, request, queryset):
    users = [item.user for item in queryset]
    for user in users:
        user.is_active = True
        user.save()
        
def reset_passwords(modeladmin, request, queryset):
    users = [item.user for item in queryset]
    for user in users:
        user.set_password('drunkduck')
        user.save()


#
# FIELDS
#

def user_email(obj):
    return obj.user.email


def join_date(obj):
    return obj.user.date_joined


def last_login(obj):
    return obj.user.last_login


def coppa_status(obj):
    if obj.birthday:
        diff = datetime.date.today() - obj.birthday
        age = diff.days/365
        if age >= 13:
            return True
        else:
            return False
    else:
        return False


class ProfileAdmin(admin.ModelAdmin):
    
    list_display = (
        'user', 
        'ip_address', 
        user_email, 
        join_date,
        last_login,
        'verified',
        'banned',
        'frozen',
        coppa_status
    )
    exclude = ('user','comments','old_user_id',)
    search_fields = ['user__username', 'ip_address', 'user__email']
    actions = [
        ban_users, 
        unban_users, 
        freeze_users, 
        unfreeze_users,
        reset_passwords
    ]
    list_filter = [
        'verified',
        'banned',
        'frozen',
    ]
    
    def save_model(self, request, instance, form, change):
        
        instance = form.save(commit=False)
        if not instance.ip_address:
            instance.ip_address = '0.0.0.0'
        instance.save()
        return instance

admin.site.register(Profile, ProfileAdmin)

def user_with_that_ip(obj):
    
    try:
        profile = Profile.objects.get(ip_address=obj.ip_address)
        return profile.user.username
    except:
        return None

class IPBanAdmin(admin.ModelAdmin):
    
    list_display = ('ip_address', 'date_banned', user_with_that_ip,) 
    search_fields = ['ip_address']
    list_filter = ['is_banned']

admin.site.register(IPBan, IPBanAdmin)
