import re
from django import forms
from django.contrib.auth.models import User
from users.models import *
from users.country_state_tuples import *
from django.forms.extras.widgets import SelectDateWidget
import datetime
from tinymce.widgets import TinyMCE

class ProfileForm(forms.ModelForm):
    
    bio = forms.CharField(widget=TinyMCE())
    
    class Meta:
        model = Profile
        exclude = ('banned', 'frozen', 'user', 'ip_address', 'avatar', 'verified','birthday','gender', 'country', 'state', 'zip',)   

class UserForm(forms.ModelForm):
    
    class Meta:
        model = User
        exclude = ('username', 'groups', 'user_permissions', 'last_login', 'date_joined', 'is_superuser', 'is_active', 'is_staff', 'password', 'first_name', 'last_name',)
        
    def clean_email(self):
        email = self.data['email']
        if not User.objects.filter(email__iexact=email).exists():
            return email
        raise forms.ValidationError('Email address is already registered.')

class RegistrationForm(forms.Form):
    
    username = forms.CharField(label='Username', max_length=30)
    email = forms.EmailField(label='Email')
    email2 = forms.EmailField(label='Comfirm Email')
    birthday = forms.DateField(label='Birthday', widget=SelectDateWidget(years=[(datetime.date.today().year - 100) + i for i in xrange(100)]))
    password1 = forms.CharField(label='Password', widget=forms.PasswordInput())
    password2 = forms.CharField(label='Password (Again)', widget=forms.PasswordInput())
    gender = forms.ChoiceField(choices=GENDERS, label='Gender')
    country = forms.ChoiceField(choices=COUNTRIES, label='Country', required=False)
    state = forms.ChoiceField(choices=STATES, label='State', required=False)
    zip = forms.CharField(label='Zip', required=False)
    agree = forms.BooleanField()
    
    def clean_agree(self):
        
        if 'agree' in self.data:
            return self.data['agree']
        raise forms.ValidationError('You must agree to the terms of service.')
    
    def clean_password2(self):
        
        if 'password1' in self.data:
            password1 = self.data['password1']
            password2 = self.data['password2']
            if password1 == password2:
                return password2
        raise forms.ValidationError('Passwords do not match.')
    
    def clean_email2(self):
        
        if 'email' in self.data:
            email = self.data['email']
            email2 = self.data['email2']
            if email == email2:
                return email2
        raise forms.ValidationError('Email addresses do not match.')
    
    def clean_username(self):
        username = self.data['username'].strip()
        if not User.objects.filter(username=username).exists():
            return username
        raise forms.ValidationError('Username is already taken.')
    
    def clean_email(self):
        email = self.data['email']
        if not User.objects.filter(email=email).exists():
            return email
        raise forms.ValidationError('Email address is already registered.')
    
class ResetPasswordForm(forms.Form):
    
    email = forms.EmailField(label='Email')
    
    def clean_email(self):
        email = self.data['email']
        if User.objects.filter(email=email).exists():
            return email
        raise forms.ValidationError('That email address is not currently registered.')
        
class ChangePasswordForm(forms.Form):
    
    oldpass = forms.CharField(label='Old Password', widget=forms.PasswordInput())
    newpass1 = forms.CharField(label='New Password', widget=forms.PasswordInput())
    newpass2 = forms.CharField(label='New Password (Again)', widget=forms.PasswordInput())
    username = forms.CharField(widget=forms.HiddenInput())
    
    def clean_oldpass(self):
        
        oldpass = self.data['oldpass']
        username = self.data['username']
        try:
            user = User.objects.get(username=username)
        except User.DoesNotExist:
            raise forms.ValidationError('Something is wrong with your user. Please contact an administrator.')
        if user.check_password(self.data['oldpass']):
            return oldpass
        raise forms.ValidationError('The old password does not match our records.')
    
    def clean_newpass2(self):
        
        if 'newpass2' in self.data:
            newpass1 = self.data['newpass1']
            newpass2 = self.data['newpass2']
            if newpass1 == newpass2:
                return newpass2
        raise forms.ValidationError('Passwords do not match.')
    
class ChangeAvatarForm(forms.ModelForm):
    class Meta:
        model = Profile
        fields = ('avatar',)
