from django.contrib.auth.models import User
from django.db import models
from django.db.models.signals import post_save, pre_save
from comments.models import Comment
from users.country_state_tuples import *
from PIL import Image
from images import *
from StringIO import StringIO
from django.core.files.uploadedfile import InMemoryUploadedFile
import settings

GENDERS = (
    ('m', 'male'),
    ('f', 'female'),
)

RELATIONSHIPS = (
    (-1, 'block'),
    (0, 'none'),
    (1, 'pending'),
    (2, 'friends'),
)

class UserRelationship(models.Model):
    
    user1 = models.ForeignKey(User, related_name='relationship_to')
    user2 = models.ForeignKey(User, related_name='relationship_from')
    status = models.IntegerField(choices=RELATIONSHIPS, default=0)
    date_added = models.DateTimeField(auto_now_add=True)
    date_modified = models.DateTimeField(auto_now=True)
        
def get_relationship(user1, user2):
    
    relationship = UserRelationship.objects.filter(user1=user1, user2=user2)
    if not relationship:
        relationship = UserRelationship.objects.filter(user1=user2, user2=user1)
    if relationship:
        return relationship[0]
    return None

def get_relationship_status(user1, user2):
    
    relationship = get_relationship(user1, user2)
    if relationship:
        return relationship.status
    return 0

def change_relationship_status(user1, user2, status):
    
    if user1 == user2:
        return
    relationship = get_relationship(user1, user2)
    if not relationship:
        relationship = UserRelationship.objects.create(user1=user1, user2=user2)
    relationship.status = status
    relationship.save()
    
def generate_verify_code(user):

    import hashlib
    from datetime import datetime
    salt = '#!#!Duck13'
    code = hashlib.md5(user.username + str(user.email) + str(datetime.now()) + salt).hexdigest()
    return code

class Email(models.Model):
    user = models.ForeignKey(User, related_name='user_email')
    email = models.EmailField()
    code = models.CharField(max_length=255)
    is_verified = models.BooleanField(default=False)

    def __unicode__(self):
        return self.user.username + ' verification'

def profile_avatar_upload(instance, filename):
    
    ext = filename.split('.')[-1].lower()
    avatar_path = settings.AWS_ROOT + instance.user.username + '/thumb/avatar.' + ext
    return avatar_path

class Profile(models.Model):
    
    user = models.OneToOneField(User, related_name='profile')
    online = models.BooleanField(default=False)
    birthday = models.DateField(null=True)
    verified = models.BooleanField(default=False, blank=True)
    banned = models.BooleanField(default=False, blank=True)
    frozen = models.BooleanField(default=False, blank=True)
    avatar = models.ImageField(upload_to=profile_avatar_upload, null=True, blank=True, max_length=750)
    gender = models.CharField(choices=GENDERS, max_length=1, null=True, default='m')
    ip_address = models.CharField(max_length=15, null=True, blank=True, default='0.0.0.0')
    bio = models.TextField(null=True, blank=True)
    country = models.CharField(choices=COUNTRIES, max_length=2, null=True, blank=True, default='US')
    state = models.CharField(choices=STATES, max_length=2, null=True, blank=True, default='')
    zip = models.IntegerField(max_length=5, null=True, blank=True)
    comments = models.ManyToManyField(Comment, null=True, blank=True)
    trophy_string = models.CharField(max_length=100, null=True, blank=True)
    old_user_id = models.IntegerField(null=True, blank=True)
    
    def __unicode__(self):
        
        return self.user.username
    
    def change_password(self, newpass):
        
        self.user.set_password(newpass)
        self.user.save()
    
    def reset_password(self):

        import string
        from random import sample
        password = sample(string.letters.lower() + string.digits, 8)
        password = ''.join(password)
        self.user.set_password(password)
        self.user.save()
        # send password to user
        from django.core.mail import send_mail
        send_mail('DrunkDuck Password Reset', 'Your new password is: ' + password, 'support@drunkduck.com', [self.user.email], fail_silently=False)
        

class IPBan(models.Model):
    
    ip_address = models.CharField(max_length=15)
    is_banned = models.BooleanField(default=True)
    date_banned = models.DateTimeField(auto_now_add=True)
    date_modified = models.DateTimeField(auto_now=True)
    
    class Meta:
        
        verbose_name = "Banned IP"
        verbose_name_plural = "Banned IPs"


def profile_creator(sender, **kwargs):
    if kwargs['created']:
        Profile.objects.create(user=kwargs['instance'])

def avatar_prep(sender, **kwargs):

    try:
        im = Image.open(kwargs['instance'].avatar)
    except:
        return
    im, resized = rescale(im, 100, 100)
    if not resized:
        return
    io = StringIO()
    filename = kwargs['instance'].avatar.name.split('/')[-1]
    ext = filename.split('.')[-1]
    if ext == 'jpg':
        ext = 'jpeg'
    im.save(io, format=ext.upper())
    file = InMemoryUploadedFile(io, None, 'avatar.' + ext, 'image/' + ext, io.len, None)
    kwargs['instance'].avatar = file

post_save.connect(profile_creator, sender=User, dispatch_uid="profile_creator_id")
pre_save.connect(avatar_prep, sender=Profile, dispatch_uid="prep_avatar")
