import os
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.db.models import Q
from django.http import HttpResponseRedirect, HttpResponseNotFound, HttpResponse
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from users.forms import *
from users.models import *
from corp.models import *
from uservideos.forms import *
from uservideos.models import *
from forums.models import *
from forums.forms import *
from requests.models import *
import datetime
from django.core.mail import EmailMultiAlternatives

def check_username(request):
    
    response_str = "false"
    if request.is_ajax():
        username = request.GET.get('username').strip()
        if os.path.exists('/users/' + username):
            response_str = "true"
        if not User.objects.filter(username__iexact=username).exists():
            response_str = "true"
        return HttpResponse(response_str)
    
def check_email(request):
    
    response_str = "false"
    if request.is_ajax():
        email = request.GET.get('email').strip()
        if not User.objects.filter(email__iexact=email).exists():
            response_str = "true"
        return HttpResponse(response_str)

def favorites(request):
    
    favorites = request.user.favorite_comics.all()
    sort = ''
    if 'sort' in request.GET:
        sort = request.GET['sort']
        if  sort == 'alpha':
            favorites = favorites.order_by('title')
        elif sort == 'update':
            favorites = favorites.order_by('-date_modified')
    return render_to_response('users/profile/favorites.html', {'favorites':favorites, 'sort':sort}, context_instance=RequestContext(request))

def forums(request):
    
    edit = Signature.objects.filter(user=request.user).exists()
    if request.method == "POST":
        if edit:
            form = SignatureForm(request.POST, instance=request.user.signature)
            if form.is_valid():
                form.save()
        else:
            form = SignatureForm(request.POST)
            if form.is_valid():
                signature = form.save(commit=False)
                signature.user = request.user
                signature.save()
    else:
        if edit:
            form = SignatureForm(instance=request.user.signature)
        else:
            form = SignatureForm()
    return render_to_response('users/profile/forums.html', {'form':form}, context_instance=RequestContext(request))

def verify_email(request):
    code = request.GET.get('c', None)

    if code:
        email = Email.objects.get(code=code)
        email.is_verified = True
        email.save()

        return render_to_response('users/email_confirm.html', {}, context_instance=RequestContext(request))
    else:
        return HttpResponseNotFound()


def signup(request):
    
    if request.user.is_authenticated():
        return HttpResponseRedirect('/')
    try:
        privacy = Corp.objects.get().signup
    except:
        privacy = None
    if request.method == "POST":
        form = RegistrationForm(request.POST)
        if form.is_valid():
            user = User.objects.create_user(
                username=form.cleaned_data['username'],
                password=form.cleaned_data['password1'],
                email=form.cleaned_data['email']
            )
            user.profile.birthday = form.cleaned_data['birthday']
            user.profile.save()

            code = generate_verify_code(user)
            verify_url = 'http://www.drunkduck.com/user/verify_email/?c=' + code

            # generate signup email
            subject, from_email, to = 'Signup Confirmation Code', 'support@drunkduck.com', user.email
            text_content = 'Please follow this link to confirm your signup: %s' % (verify_url,)
            html_content = 'Please follow this link to confirm your signup: <a href="%s">%s</a>' % (verify_url, verify_url)
            msg = EmailMultiAlternatives(subject, text_content, from_email, [to])
            msg.attach_alternative(html_content, "text/html")
            msg.send()

            Email.objects.create(user=user, email=user.email, code=code)

            return HttpResponseRedirect('/user/signup/complete/')
#	else:
#	    print form.errors
    else:
        form = RegistrationForm()
        form.initial['country'] = 'US'
        form.initial['birthday'] = str(datetime.date.today().year - 13) + '-01-01'
    return render_to_response('users/form_signup.html', {'form':form, 'privacy':privacy}, context_instance=RequestContext(request))

def signup_complete(request):
    
    return render_to_response('users/form_signup_complete.html', context_instance=RequestContext(request))
    
def reset(request):
    
    if request.user.is_authenticated():
        return HttpResponseRedirect('/')
    if request.method == "POST":
        form = ResetPasswordForm(request.POST)
        if form.is_valid():
            user = User.objects.get(email=form.cleaned_data['email'])
            user.profile.reset_password()
            return HttpResponseRedirect('/user/reset/complete/')
    else:
        form = ResetPasswordForm()
    return render_to_response('users/form_reset_password.html', {'form':form}, context_instance=RequestContext(request))

def reset_complete(request):

    return render_to_response('users/form_reset_password_complete.html', context_instance=RequestContext(request))
    
def display(request, username=None):
    
    variables = {}
    if not username:
        if not request.user.is_authenticated():
            return HttpResponseNotFound()
        user = request.user
    else:
        try:
            user = User.objects.get(username__iexact=username)
        except:
            return HttpResponseNotFound()
        if request.user.is_authenticated():
            variables['friend_status'] = get_relationship_status(request.user, user)
    if user.profile.banned:
        return HttpResponseNotFound()
    variables['profile'] = user.profile
    variables['videos'] = UserVideo.objects.filter(user=user, is_deleted=False)
    if request.user == user:
        variables['add_video_form'] = UserVideoForm()
    # GET TOPICS
    topics = Topic.objects.filter(user=user, is_deleted=False)
    if request.user != user and not request.user.is_staff:
        topics = topics.filter(forum__is_private=False)
    variables['topics'] = topics
    variables['assisted_comics'] = user.assisted_comics.filter(is_deleted=False)
    variables['recommended_comics'] = user.recommended_comics.filter(is_deleted=False)
    variables['comics'] = user.comics.filter(is_deleted=False)
    variables['view'] = 'profile'
    # GET FRIENDS
    variables['friends'] = UserRelationship.objects.filter(Q(status__gte=2), Q(user1=user) | Q(user2=user)).order_by('?')
    return render_to_response('users/display.html', variables, context_instance=RequestContext(request)) 

@login_required
def edit(request):
    
    message = False
    change_avatar_form = ChangeAvatarForm()
    change_password_form = ChangePasswordForm()
    profile_form = ProfileForm(instance=request.user.profile)
    user_form = UserForm(instance=request.user)
    if request.method == "POST":
        if 'save_info' in request.POST:
            profile_form = ProfileForm(request.POST, instance=request.user.profile)
            if profile_form.is_valid():
                profile_form.save()
        elif 'change_email' in request.POST:
            user_form = UserForm(request.POST, instance=request.user)
            if user_form.is_valid():
                user_form.save()
        elif 'change_password' in request.POST:
            change_password_form = ChangePasswordForm(request.POST)
            if change_password_form.is_valid():
                request.user.profile.change_password(change_password_form.cleaned_data['newpass1'])
                change_password_form = ChangePasswordForm()
                message = 'Password changed successfully.'
        elif 'change_avatar' in request.POST:
            change_avatar_form = ChangeAvatarForm(request.user.profile)
            # change avatar
    change_password_form.fields['username'].initial = request.user.username
    variables = {
        'profile_form':profile_form, 
        'user_form':user_form, 
        'change_password_form': change_password_form,
        'change_avatar_form': change_avatar_form,
        'message': message
    }
    return render_to_response('users/edit.html', variables, context_instance=RequestContext(request)) 

@login_required
def request_friendship(request, username):
    
    user = User.objects.filter(username=username)
    if user:
        user = user[0]
        change_relationship_status(request.user, user, 1)
        Request.objects.create(user=user, request_type=0, content_object=request.user)
    return HttpResponseRedirect('/user/' + username + '/')

@login_required
def remove_friendship(request, username):

    user = User.objects.filter(username=username)
    if user:
        user = user[0]
        change_relationship_status(request.user, user, 0)
    return HttpResponseRedirect('/user/' + username + '/')
        
@login_required
def upload_avatar(request):
    if request.method == "POST":
        form = ChangeAvatarForm(request.POST, request.FILES)
        if form.is_valid():
            data = form.cleaned_data
            profile = request.user.get_profile()
            profile.avatar = data['avatar']
            profile.save()
#            form.save()
            return HttpResponseRedirect(request.META['HTTP_REFERER'])
    return HttpResponseNotFound()
