from django.db import models
from django.contrib.auth.models import User
from comments.models import Comment

from ratings.models import Like
from django.template.defaultfilters import slugify



def video_thumbnail_upload(instance, filename):

    return 'videos/thumb/' + filename

class Video(models.Model):
    
    is_live = models.BooleanField(default=False)
    is_featured = models.BooleanField(default=False)
    is_in_carousel = models.BooleanField(default=False)
    title = models.CharField(max_length=255)
    subtitle = models.CharField(max_length=200, default='')
    carousel_text = models.CharField(max_length=25, blank=True, null=True)
    carousel_subtext = models.CharField('Carousel Sub-text',max_length=15,blank=True,null=True)
    duration = models.CharField("Video Duration",max_length=20,blank=True,null=True)
    director = models.CharField("Director",max_length=255,blank=True,null=True)
    producer = models.CharField("Producer / Studio",max_length=255,blank=True,null=True)
    pub_date = models.DateTimeField('Published Date',blank=True,null=True)
    url = models.URLField(null=True, blank=True)
    embed = models.TextField('Embed Code', null=True, blank=True)
    description = models.TextField('details')
    date_added = models.DateTimeField(auto_now_add=True)
    user = models.ForeignKey(User, related_name='admin_videos', null=True, blank=True)
    comments = models.ManyToManyField(Comment, null=True, blank=True)
    likes = models.ManyToManyField(Like, null=True, blank=True)
    thumbnail = models.ImageField(upload_to=video_thumbnail_upload, null=True, blank=True)
    tags = models.TextField('Tags',help_text='Keywords to help searching for content.', null=True, blank=True)
    wowio = models.BooleanField('Available to WOWIO?',default=False)
    wevolt = models.BooleanField('Available to WEVolt?',default=False)
    #drunkduck = models.BooleanField('Available to DrunkDuck?',default=False)
    syndicate = models.BooleanField('Available for Syndication?',default=False)
    slug = models.SlugField(
#        unique_for_date='pub_date',
        help_text='Automatically built from the title. -- DO NOT MODIFY',
        blank=True,
        null=True
    )
    
    def __unicode__(self):
    
        return self.title
    
    class Meta:
        
        ordering = ('-date_added',)

    def save(self, *args, **kwargs):
        self.slug = slugify(self.title)[:50]
        super(Video, self).save(*args, **kwargs)




