from django.db.models import Count
from django.contrib.auth import authenticate, login, logout
from django.shortcuts import render_to_response, render
from django.http import HttpResponseRedirect, HttpResponseNotFound
from django.template import RequestContext
from django.conf import settings
from comics.models import *
from videos.models import *
from news.models import *
from search.forms import *
from django.contrib.auth.forms import AuthenticationForm
from datetime import datetime
from django.contrib.auth.models import User

def home_page(request):
    
    # Featured video
    try:
        featured_video = Video.objects.get(is_featured=True,is_live=True)
    except:
        featured_video = None

	# Featured news
    news = Article.objects.filter(urgent=False, is_live=True, date_live__lte=datetime.now()).order_by("-date_live")[:5]

    # Featured comics
    featured = FeaturedComic.objects.filter(comic__is_deleted=False, date_featured__lte=datetime.now()).order_by('-date_featured')[:15]

    # Select top ten comics
    topten = Comic.live.order_by('-seven_day_visits')[:10]

    # Build  Random comics list
    quails = Comic.live.order_by('?')[:10]

    # latest updates
    latest = Comic.live.order_by('-date_modified')[:10]
    
    variables = {
        'view':'home',
        'featured': featured,
        'video': featured_video,
        'news': news,
        'topten': topten,
        'quails': quails,
        'latest': latest,
        'search_form':SearchForm(),
    }
    return render(request, 'homepage/index.html', variables)

def do_login(request):

    next = '/'
    form = AuthenticationForm()
    error = ''
    if request.method == 'POST':
        if request.POST.get('next', False):
            next = request.POST['next']
        else:
            next = request.META.get('HTTP_REFERER', '/')
        try:
            user = User.objects.get(username__iexact=request.POST['username'])
            user = authenticate(username=user.username, password=request.POST['password'])
            login(request, user)
            user.profile.online = True
            user.profile.save()
            return HttpResponseRedirect(next)
        except:
            pass
        error = 'Your username and password do not match our records.'
    return render(request, 'users/login.html', {'form':form, 'next':next, 'error':error})

def user_logout(request):
    
    request.user.profile.online = False
    request.user.profile.save()
    logout(request)
    return HttpResponseRedirect("/")

def favorites_list(request):
    
    user_favorites = request.user.favorite_comics.filter(is_deleted=False)
    if request.GET['sort']:
        user_favorites = user_favorites.order_by(request.GET['sort'])
    return render_to_response('user-side-panel/favorites_list.html', {'user_favorites':user_favorites}, context_instance=RequestContext(request))

def comics_list(request):
    
    user_comics = request.user.comics.filter(is_deleted=False)
    if request.GET['sort']:
        user_comics = user_comics.order_by(request.GET['sort'])
    return render_to_response('user-side-panel/comics_list.html', {'user_comics':user_comics}, context_instance=RequestContext(request))
    
def ten_list(request, type):
    
    comics = Comic.objects.filter(is_deleted=False, pages__post_date__lte=datetime.now()).annotate(count=Count('title'))
    if type == 'topten':
        comics = comics.order_by('-seven_day_visits')
    elif type == 'quail':
        comics = comics.order_by('?')
    elif type == 'latest':
        comics = comics.order_by('-date_modified')
    if 'style' in request.GET:
        comics = comics.filter(style__in=request.GET.getlist('style'))
    if 'genre' in request.GET:
        comics = comics.filter(genre__in=request.GET.getlist('genre'))
    if 'tone' in request.GET:
        comics = comics.filter(tone__in=request.GET.getlist('tone'))
    if 'type' in request.GET:
        comics = comics.filter(type__in=request.GET.getlist('type'))
    if 'rating' in request.GET:
        comics = comics.filter(rating__in=request.GET.getlist('rating'))
    if comics:
        comics = comics[:10]
    else:
        return HttpResponseNotFound()
    return render_to_response('homepage/ten-lists/list.html', {'comics':comics}, context_instance=RequestContext(request))
    
    
